------------------------------------------------------------------------------------------------------------------------
-- utilities.lua - define common functions and classes
-- wow addon by Applesinych [k.shabordin@gmail.com]
------------------------------------------------------------------------------------------------------------------------
local GetScreenWidth,GetScreenHeight,CreateFrame,GetQuestDifficultyColor,UnitPowerType,format = 
      GetScreenWidth,GetScreenHeight,CreateFrame,GetQuestDifficultyColor,UnitPowerType,string.format

local DB = SNT_INFO.DB  -- get globals
local function error(msg) ChatFrame1:AddMessage("|cff33ff99snt error:|r "..msg) end



local util = {}
   function util.round(val,num) return (floor(val*10*num))/(10*num) end
   function util.rgbt2str(col) local r,g,b = unpack(col) return format("|cff%02x%02x%02x",r*255,g*255,b*255) end
   function util.rgb2str(r,g,b) return format("|cff%02x%02x%02x",r*255,g*255,b*255) end
   function util.money(val) return format("|cffffd700%d|r.|cffbbbbdd%d|r.|cffeda55f%d|r",mod(val/10000,10000),mod(val/100,100),mod(val,100)) end
   
   function util.level(val)
      local lcol = GetQuestDifficultyColor(val) 
      if not lcol then return 1,1,1 end
      return lcol.r,lcol.g,lcol.b
   end
   function util.level_s(val) return util.rgb2str(util.level(val))..val.."|r" end
   function util.class(val) 
      val = strupper(val)
      local col = DB.CLASS[val]
      if(col) then return util.rgbt2str(DB.CLASS[val]) end
      return "|cff000000" 
   end
   function util.gradient(val,bad,good)
      local percent,r,g
      if (good > bad) then percent = val/(good-bad)
      else percent = 1-val/(bad-good) end
      if (percent > 1) then percent = 1 end
      if (percent < 0) then percent = 0 end
      if(percent < 0.5) then r,g = 1,2*percent   else  r,g = (1-percent)*2,1 end
      return r,g,0
   end

   function util.gradient_s(val,bad,good)
      return util.rgb2str(util.gradient(val,bad,good))..val.."|r"
   end
   
   
   function util.memory(val,bad,good)
      if val > 1024 then return util.gradient_s(util.round(val/1024,1),bad,good).." mb"
   else return util.gradient_s(util.round(val,1),bad*1024,good*1024).." kb" end
end

  --------------------------------------------------------------------------------------------------------------------  
  
   function util.movable(frame)
      frame:EnableMouse(true)
      frame:SetMovable(true)               
      frame:SetScript("OnMouseDown",function(self) self:StartMoving() end)         
      frame:SetScript("OnMouseUp",  function(self) self:StopMovingOrSizing() self.moved = true end)
   end
   function util.movable_if(frame)
      frame:EnableMouse(true)
      frame:SetMovable(true)               
      frame:SetScript("OnMouseDown",function(self) if(not DB.lock) then self:StartMoving() end end)         
      frame:SetScript("OnMouseUp",  function(self) if(not DB.lock) then self:StopMovingOrSizing() self.moved = true end end)
   end
 
   function util.colorize(frame,bk_col,br_col)
      frame:SetBackdropColor(unpack(bk_col)) 
      frame:SetBackdropBorderColor(unpack(br_col))
   end
   function util.place(frame,x1,y1,x2,y2)
      local lt_x,lt_y
      if(x1<x2) then lt_x = x1 else lt_x = x2 end 
      if(y1<y2) then lt_y = y1 else lt_y = y2 end
      frame:SetWidth(abs(x2-x1))
      frame:SetHeight(abs(y2-y1))
      frame:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",lt_x,lt_y)
   end
   function util.point (frame,w,h,x,y)
      frame:ClearAllPoints()
      frame:SetWidth(w);frame:SetHeight(h)
      frame:SetPoint("CENTER",UIParent,"CENTER",x,y) 
   end
   function util.set_backdrop(frame,background,border)
      frame:SetBackdrop({bgFile = background, edgeFile = border,edgeSize = DB.edge,
                        insets = {left = DB.inset, right = DB.inset, top = DB.inset, bottom = DB.inset}})
   end   


--- global line class --------------------------------------------------------------------------------------------------
--
-- l = line:new(type,x,y,th,lenght,color,movable)
--     type    - "v" or "h" for vertical or horizontal line  
--     x,y     - begin line point
--     tn      - pixel thickness of line
--     lenght  - pixel lenght of line
--     color   - color of line {r,g,b} or {r,g,b,a} table
--     movable - boolean. its line is movable?  
--     return  - line object
------------------------------------------------------------------------------------------------------------------------
local line = {}
   function line:new(ltype,x,y,th,lenght,color,movable)
      local instance = {}
      local h,w 
      instance.frame = CreateFrame("frame",nil,UIParent)
      if(ltype == "v")     then h = lenght; w = th;   instance.frame:SetBackdrop({bgFile = SNT_INFO.DB.vline})
      elseif(ltype == "h") then h = th; w = lenght    instance.frame:SetBackdrop({bgFile = SNT_INFO.DB.hline})
      else error("incorrect line:new() type parameter") end
      instance.frame:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",x,y)
      instance.frame:SetBackdropColor(unpack(color))   
      instance.frame:SetWidth(w)
      instance.frame:SetHeight(h)
      instance.frame:Show()
      if(movable) then util.movable(instance.frame) end   
      instance.frame:SetFrameStrata("LOW")
      setmetatable(instance,self)   
      self.__index   = self
      return instance
   end
----global panel class -------------------------------------------------------------------------------------------------
--
-- p = panel:new()            - create abstract panel
-- p:place(x1,y1,x2,y2)       - place panel on absolute coordinates 
-- p:decor(parent)            - decor parent frame, parent - name of need to decorate frame
-- p:show()                   - show panel
-- p:hide()                   - hide panel
-- p:do_movable()             - do movable panel
-- p:set_text(text,dummy)     - set text to panel (create text subframe on first call), if dummy exist - no resize 
-- p:colorize(bk_col,br_col)  - colorize panel with background and border colors
------------------------------------------------------------------------------------------------------------------------
local panel = {}

   function panel:colorize(...)        util.colorize(self.frame,...)       end
   function panel:place(...)           util.place(self.frame,...)          end
   function panel:do_movable()         util.movable(self.frame)            end
   function panel:do_movable_if()      util.movable_if(self.frame)         end
   function panel:set_backdrop(...)    util.set_backdrop(self.frame,...)   end
   function panel:show()               self.frame:Show()                   end
   function panel:hide()               self.frame:Hide()                   end
   function panel:point(w,h,x,y)       util.point(self.frame,w,h,x,y)      end
------------------------------------------------------------------------------------------------------------------------   
   function panel:decor(parent)                                         
      if(parent) then
         self.frame:SetParent(parent)   
         self.frame:SetPoint("BOTTOMLEFT",parent,"BOTTOMLEFT",-6,-6)
         self.frame:SetPoint("TOPRIGHT",parent,"TOPRIGHT",6,6)
         self.frame:SetFrameLevel(parent:GetFrameLevel())
         parent:SetFrameLevel(parent:GetFrameLevel()+1)
      end   
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:add_text(text,resize)                                         
      local wp = 8
         if(not self.frame.txt) then      
            self.frame.txt=self.frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
            self.frame.txt:SetFont(DB.font,DB.fontsize,nil)
            self.frame.txt:SetShadowOffset(1,-1)
            self.frame.txt:SetTextColor(unpack(DB.font_col))
         end  
      if(resize) then 
         self.frame:SetWidth(self.frame.txt:GetStringWidth()+wp) 
      end
      self.frame.txt:SetAllPoints(self.frame)
      self.frame.txt:SetText(text)
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:add_update(uf,timing)
      local timer = 0
      self.frame:SetScript("OnUpdate", function(_,elapsed)
                                          timer = timer + elapsed
                                          if(timer >= timing) then uf();timer = 0 end   end)
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:add_tooltip(tt_f)
      local anchor = self.frame
      anchor:SetScript("OnEnter",function(self)
                                    anchor:SetAlpha(1)      
                                    GameTooltip:SetOwner(self,"ANCHOR_BOTTOMRIGHT")
                                    tt_f()
                                    GameTooltip:Show()
                                   end)              
      anchor:SetScript("OnLeave", function() anchor:SetAlpha(0.9) GameTooltip:Hide() end)
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:set_click(cl_f)
      local anchor = self.frame
      anchor:RegisterForClicks("AnyUp")
      anchor:SetScript("OnClick",function() cl_f() end)              
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:add_event(event,ev_f)
      self.frame:RegisterEvent(event)
      self.events[event] = ev_f
      self.frame:SetScript("OnEvent",function(_,ev,...) self.events[ev](self)  end)
   end
------------------------------------------------------------------------------------------------------------------------   
   function panel:new(...)                      
      local name
      if(select("#",...)>0) then name = select(1,...) end                                                                           
      local instance    = {}
      instance.events   = {}
      instance.frame = CreateFrame("button",name,UIParent)  
      instance.frame:SetBackdrop({bgFile = DB.background, edgeFile = DB.border,edgeSize = DB.edge,tile = true,tileSize = 32,
                          insets = {left = DB.inset, right = DB.inset, top = DB.inset, bottom = DB.inset}})
      instance.frame:SetBackdropColor(unpack(DB.BG_col)) 
      instance.frame:SetBackdropBorderColor(unpack(DB.BR_col))
      instance.frame:SetFrameStrata("BACKGROUND")
      setmetatable(instance,self)   
      self.__index   = self
      return instance
   end


--- set global scope for usable structures ----------------------------------------------------------------------------   

   SNT_INFO.util       = util
   SNT_INFO.line       = line
   SNT_INFO.panel      = panel
   SNT_INFO.bar        = bar

    




